
import os
import json
import errno


class ServiceConfig(object):
    """
    Base class for reading and writing service configuration
    files to disk.
    """
    def __init__(self, config_file):
        self.config_path = os.path.expanduser(config_file)
        self.data = self._get_service_config()

    def save(self, data):
        """
        Save the given configuration data.
        """
        self.data = data

    def _get_service_config(self):
        """
        Reads in config file of UAA credential information
        or generates one as a side-effect if not yet
        initialized.
        """
        # Should work for windows, osx, and linux environments
        if not os.path.exists(self.config_path):
            try:
                os.makedirs(os.path.dirname(self.config_path))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise

            return {}

        with open(self.config_path, 'r') as data:
            return json.load(data)

    def _write_service_config(self):
        """
        Will write the config out to disk.
        """
        with open(self.config_path, 'w') as output:
            output.write(json.dumps(self.data, sort_keys=True, indent=4))

    def __del__(self):
        """
        Destructor to write out service config to disk at termination
        of a running process.
        """
        self._write_service_config()




import os
import yaml


class Manifest(object):
    """
    Cloud Foundry utilizes a MANIFEST.yml file as the source
    of application configuration.  As we setup services and
    run our applications the manifest is a place to store
    important configuration details.
    """
    def __init__(self, manifest_path, app_name='my-predix-app'):
        self.manifest_path = os.path.expanduser(manifest_path)
        self.app_name = app_name

        # Read or Generate a manifest file
        if os.path.exists(self.manifest_path):
            manifest = self.read_manifest()
        else:
            manifest = self.create_manifest()

        # Probably always want manifest loaded into environment
        self.set_os_environ()

    def read_manifest(self):
        """
        Read an existing manifest.
        """
        with open(self.manifest_path, 'r') as input_file:
            self.manifest = yaml.safe_load(input_file)
            if 'env' not in self.manifest:
                self.manifest['env'] = {}
            if 'services' not in self.manifest:
                self.manifest['services'] = []

            input_file.close()

    def create_manifest(self):
        """
        Create a new manifest and write it to
        disk.
        """
        self.manifest = {}
        self.manifest['applications'] = [{'name': self.app_name}]
        self.manifest['services'] = []
        self.manifest['env'] = {}

        self.write_manifest()

    def write_manifest(self):
        """
        Write manifest to disk.
        """
        with open(self.manifest_path, 'w') as output_file:
            yaml.safe_dump(self.manifest, output_file,
                    default_flow_style=False, explicit_start=True)
            output_file.close()

    def add_env_var(self, key, value):
        """
        Add the given key / value as another environment
        variable.
        """
        self.manifest['env'][key] = value

    def add_service(self, service_name):
        """
        Add the given service to the manifest.
        """
        if service_name not in self.manifest['services']:
            self.manifest['services'].append(service_name)

    def set_os_environ(self):
        """
        Will load any environment variables found in the
        manifest file into the current process for use
        by applications.

        When apps run in cloud foundry this would happen
        automatically.
        """
        for key in self.manifest['env'].keys():
            os.environ[key] = self.manifest['env'][key]

    def get_client_id(self):
        """
        Return the client id that should have all the
        needed scopes and authorities for the services
        in this manifest.
        """
        if 'PREDIX_APP_CLIENT_ID' not in self.manifest['env']:
            raise ValueError('UAA client id must be added to manifest.')

        return self.manifest['env']['PREDIX_APP_CLIENT_ID']

    def get_client_secret(self):
        """
        Return the client secret that should correspond with
        the client id.
        """
        if 'PREDIX_APP_CLIENT_SECRET' not in self.manifest['env']:
            raise ValueError('UAA client secret must be added to manifest.')

        return self.manifest['env']['PREDIX_APP_CLIENT_SECRET']




import os

import predix.app
import predix.security.uaa
import predix.admin.service


class Asset(object):
    """
    Asset Service provides asset management functionality.
    """
    def __init__(self, plan_name=None, name=None, uaa=None, *args, **kwargs):
        super(Asset, self).__init__(*args, **kwargs)
        self.service_name = 'predix-asset'
        self.plan_name = plan_name or 'Free'
        self.service = predix.admin.service.PredixService(self.service_name,
                self.plan_name, name=name, uaa=uaa)

    def exists(self):
        """
        Returns whether or not this service already exists.
        """
        return self.service.exists()

    def create(self):
        """
        Create an instance of the Asset Service with the typical
        starting settings.
        """
        self.service.create()
        os.environ['PREDIX_ASSET_URI'] = self.service.settings.data['uri']
        os.environ['PREDIX_ASSET_ZONE_ID'] = self.get_zone_id()

    def grant_client(self, client_id):
        """
        Grant the given client id all the scopes and authorities
        needed to work with the asset service.
        """
        zone = self.service.settings.data['zone']['oauth-scope']

        scopes = ['openid', zone]

        authorities = ['uaa.resource', zone]

        self.service.uaa.uaac.update_client_grants(client_id, scope=scopes,
                authorities=authorities)

        return self.service.uaa.uaac.get_client(client_id)

    def get_zone_id(self):
        """
        Returns the Predix-Zone-Id used for this service.
        """
        return self.service.settings.data['zone']['http-header-value']

    def add_to_manifest(self, manifest_path):
        """
        Add details to the manifest that applications using
        this service may need to consume.
        """
        manifest = predix.app.Manifest(manifest_path)

        # Add this service to list of services
        manifest.add_service(self.service.name)

        # Add environment variables
        manifest.add_env_var('PREDIX_ASSET_URI',
                self.service.settings.data['uri'])
        manifest.add_env_var('PREDIX_ASSET_ZONE_ID', self.get_zone_id())

        manifest.write_manifest()




import os
import json
import errno
import base64
import logging
import requests
import datetime
import dateutil.parser

import predix.app

class UserAccountAuthentication(object):
    """
    The UAA service manages user account authorization and access control for
    Predix service calls through client credentials captured in a bearer token.

    Useful documentation about interacting with API here:
    https://docs.cloudfoundry.org/api/uaa
    """
    def __init__(self):

        self.uri = os.environ.get('PREDIX_UAA_URI')
        if not self.uri:
            raise ValueError("PREDIX_UAA_URI environment unset")

        self.authenticated = False
        self.client = None

    def _authenticate_client(self, client, secret):
        """
        Returns response of authenticating with the given client and
        secret.
        """
        credentials = base64.b64encode(str.join(':', [client, secret]))
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Cache-Control': 'no-cache',
            'Authorization': 'Basic ' + credentials
            }
        params = {
            'client_id': client,
            'grant_type': 'client_credentials'
            }
        uri = self.uri + '/oauth/token'

        logging.debug("URI=" + str(uri))
        logging.debug("HEADERS=" + str(headers))
        logging.debug("BODY=" + str(params))

        response = requests.post(uri, headers=headers, params=params)
        if response.status_code == 200:
            return response.json()
        else:
            response.raise_for_status()

    def is_expired_token(self, client):
        """
        For a given client will test whether or not the token
        has expired.
        """
        if 'expires' not in client:
            return True

        expires = dateutil.parser.parse(client['expires'])
        if expires < datetime.datetime.now():
            return True

        return False

    def _initialize_uaa_cache(self):
        """
        If we don't yet have a uaa cache we need to 
        initialize it.  As there may be more than one
        UAA instance we index by issuer and then store
        any clients, users, etc.
        """
        try:
            os.makedirs(os.path.dirname(self._cache_path))
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise

        data = {}
        data[self.uri] = []

        return data

    def _read_uaa_cache(self):
        """
        Read cache of UAA client/user details.
        """
        self._cache_path = os.path.expanduser('~/.predix/uaa.json')
        if not os.path.exists(self._cache_path):
            return self._initialize_uaa_cache()

        with open(self._cache_path, 'r') as data:
            return json.load(data)

    def _get_client_from_cache(self, client_id):
        """
        For the given client_id return what is
        cached.
        """
        data = self._read_uaa_cache()

        # Only if we've cached any for this issuer
        if self.uri not in data:
            return

        for client in data[self.uri]:
            if client['id'] == client_id:
                return client

    def _write_to_uaa_cache(self, new_item):
        """
        Cache the client details into a cached file on disk.
        """
        data = self._read_uaa_cache()

        # Initialize client list if first time
        if self.uri not in data:
            data[self.uri] = []

        # Remove existing client record
        for client in data[self.uri]:
            if new_item['id'] == client['id']:
                data[self.uri].remove(client)

        data[self.uri].append(new_item)

        with open(self._cache_path, 'w') as output:
            output.write(json.dumps(data, sort_keys=True, indent=4))

    def authenticate(self, client_id, client_secret, use_cache=True):
        """
        Authenticate the given client against UAA.  The resulting token
        will be cached for reuse.
        """
        # We will reuse a token for as long as we have one cached
        # and it hasn't expired.
        if use_cache:
            client = self._get_client_from_cache(client_id)
            if (client) and (not self.is_expired_token(client)):
                self.authenticated = True
                self.client = client
                return

        # Let's authenticate the client
        client = {
            'id': client_id,
            'secret': client_secret
        }

        res = self._authenticate_client(client_id, client_secret)
        client.update(res)

        expires = datetime.datetime.now() + \
                  datetime.timedelta(seconds=res['expires_in'])
        client['expires'] = expires.isoformat()

        # Cache it for repeated use until expired
        self._write_to_uaa_cache(client)

        self.client = client
        self.authenticated = True

    def get_token(self):
        """
        Returns the bare access token for the authorized client.
        """
        if not self.authenticated:
            raise ValueError("Must authenticate client first.")

        # If token has expired we'll need to refresh and get a new
        # client credential
        if self.is_expired_token(self.client):
            logging.info("client token expired, will need to refresh token")
            self.authenticate(self.client['id'], self.client['secret'],
                    use_cache=False)

        return self.client['access_token']

    def get_scopes(self):
        """
        Returns the scopes for the authorized client.
        """
        if not self.authenticated:
            raise ValueError("Must authenticate client first.")

        scope = self.client['scope']
        return scope.split()

    def get_authorization_headers(self):
        """
        Returns the authorization headers with the bearer token needed for
        making calls to Predix Services protected by UAA.
        """
        return {
                'Authorization': 'Bearer ' + self.get_token()
                }

    def is_admin(self):
        """
        Test whether user is authenticated as the admin.
        """
        return self.authenticated and self.client['id'] == 'admin'

    def get_clients(self):
        """
        Returns the clients stored in the instance of UAA.
        """
        assert self.is_admin(), "Must be admin to get_clients()."

        uri = self.uri + '/oauth/clients'
        headers = self.get_authorization_headers()
        response = requests.get(uri, headers=headers)
        return response.json()['resources']

    def get_client(self, client_id):
        """
        Returns details about a specific client by the client_id.
        """
        uri = self.uri + '/oauth/clients/' + client_id
        headers = self.get_authorization_headers()
        response = requests.get(uri, headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            # Not found but don't raise
            return

    def update_client_grants(self, client_id, scope=[], authorities=[],
            replace=False):
        """
        Will extend the client with any additional scopes or
        authorities.
        """
        client = self.get_client(client_id)
        if not client:
            raise ValueError("Must first create client: '%s'" % (client_id))

        if replace:
            changes = {
                'client_id': client_id,
                'scope': scope,
                'authorities': authorities,
                }
        else:
            changes = {'client_id': client_id}
            if scope:
                changes['scope'] = client['scope']
                changes['scope'].extend(scope)

            if authorities:
                changes['authorities'] = client['authorities']
                changes['authorities'].extend(authorities)

        uri = self.uri + '/oauth/clients/' + client_id
        headers = {
            "pragma": "no-cache",
            "Cache-Control": "no-cache",
            "Content-Type": "application/json",
            "Accepts": "application/json",
            "Authorization": "Bearer " + self.get_token()
        }
        response = requests.put(uri, headers=headers, data=json.dumps(changes))

        if response.status_code == 200:
            return response
        else:
            response.raise_for_status()

    def create_client(self, client_id, client_secret, manifest=None):
        """
        Will create a new client for your application use.
        """
        assert self.is_admin(), "Must be admin to create_client()."

        # Check if client already exists
        client = self.get_client(client_id)
        if client:
            return client

        uri = self.uri + '/oauth/clients'
        headers = {
            "pragma": "no-cache",
            "Cache-Control": "no-cache",
            "Content-Type": "application/json",
            "Accepts": "application/json",
            "Authorization": "Bearer " + self.get_token()
        }
        params = {
            "client_id": client_id,
            "client_secret": client_secret,
            "scope": ["uaa.none"],
            "authorized_grant_types": ["client_credentials", "refresh_token"],
            "authorities": ["uaa.none"],
            "autoapprove": []
        }

        response = requests.post(uri, headers=headers, data=json.dumps(params))
        if response.status_code == 201:
            if manifest:
                self.add_client_to_manifest(client_id, client_secret, manifest)

            client = {
                'id': client_id,
                'secret': client_secret
                }
            self._write_to_uaa_cache(client)
            return response
        else:
            response.raise_for_status()

    def add_client_to_manifest(self, client_id, client_secret, manifest_path):
        """
        Add the given client / secret to the manifest for use in
        the application.
        """
        manifest = predix.app.Manifest(manifest_path)
        manifest.add_env_var('PREDIX_APP_CLIENT_ID', client_id)
        manifest.add_env_var('PREDIX_APP_CLIENT_SECRET', client_secret)
        manifest.write_manifest()




import unittest

import predix.security.uaa

class TestUAA(unittest.TestCase):
    def test_init(self):
        uaa = predix.security.uaa.UserAccountAuthentication()
        self.assertIsInstance(uaa, predix.security.uaa.UserAccountAuthentication)


if __name__ == '__main__':
    unittest.main()




import os
import urllib

import predix.service


class WeatherForecast(object):
    def __init__(self, *args, **kwargs):
        super(WeatherForecast, self).__init__(*args, **kwargs)

        self.uri = os.environ.get('PREDIX_WEATHER_URI')
        if not self.uri:
            raise ValueError("PREDIX_WEATHER_URI environment unset")

        self.zone_id = os.environ.get('PREDIX_WEATHER_ZONE_ID')
        if not self.zone_id:
            raise ValueError("PREDIX_WEATHER_ZONE_ID environment unset")

        self.service = predix.service.Service(self.zone_id)

    def authenticate_as_client(self, client_id, client_secret):
        self.service.uaa.authenticate(client_id, client_secret)

    def get_weather_forecast_days(self, latitude, longitude,
            days=1, frequency=1, reading_type=None):
        """
        Return the weather forecast for a given location.

            results = ws.get_weather_forecast_days(lat, long)
            for w in results['hits']:
                print w['start_datetime_local']
                print w['reading_type'], w['reading_value']

        For description of reading types:
        https://graphical.weather.gov/xml/docs/elementInputNames.php
        """
        params = {}

        # Can get data from NWS1 or NWS3 representing 1-hr and 3-hr
        # intervals.
        if frequency not in [1, 3]:
            raise ValueError("Reading frequency must be 1 or 3")

        params['days'] = days
        params['source'] = 'NWS' + str(frequency)
        params['latitude'] = latitude
        params['longitude'] = longitude

        if reading_type:
            # url encoding will make spaces a + instead of %20, which service
            # interprets as an "and" search which is undesirable
            reading_type = reading_type.replace(' ', '%20')
            params['reading_type'] = urllib.quote_plus(reading_type)

        url = self.uri + '/v1/weather-forecast-days/'
        return self.service._get(url, params=params)

    def get_weather_forecast(self, latitude, longitude, start, end,
            frequency=1, reading_type=None):
        """
        Return the weather forecast for a given location for specific
        datetime specified in UTC format.

            results = ws.get_weather_forecast(lat, long, start, end)
            for w in results['hits']:
                print w['start_datetime_local']
                print w['reading_type'], '=', w['reading_value']

        For description of reading types:
        https://graphical.weather.gov/xml/docs/elementInputNames.php
        """
        params = {}

        # Can get data from NWS1 or NWS3 representing 1-hr and 3-hr
        # intervals.
        if frequency not in [1, 3]:
            raise ValueError("Reading frequency must be 1 or 3")

        params['source'] = 'NWS' + str(frequency)
        params['latitude'] = latitude
        params['longitude'] = longitude
        params['start_datetime_utc'] = start
        params['end_datetime_utc'] = end

        if reading_type:
            # Not using urllib.quote_plus() because its using a + which is
            # being interpreted by service as an and instead of a space.
            reading_type = reading_type.replace(' ', '%20')
            params['reading_type'] = reading_type

        url = self.uri + '/v1/weather-forecast-datetime/'
        return self.service._get(url, params=params)




import os

import predix.app
import predix.admin.service


class WeatherForecast(object):
    def __init__(self, plan_name=None, name=None, uaa=None, *args, **kwargs):
        super(WeatherForecast, self).__init__(*args, **kwargs)
        self.service_name = 'us-weather-forecast'
        self.plan_name = plan_name or 'Beta'
        self.service = predix.admin.service.PredixService(self.service_name,
                self.plan_name, name=name, uaa=uaa)

    def exists(self):
        """
        Returns whether or not this service already exists.
        """
        return self.service.exists()

    def create(self):
        """
        Create an instance of the US Weather Forecast Service with
        typical starting settings.
        """
        self.service.create()

        zone = self.service.settings.data['zone']['http-header-value']
        os.environ['PREDIX_WEATHER_ZONE_ID'] = zone
        uri = self.service.settings.data['uri']
        os.environ['PREDIX_WEATHER_URI'] = uri

    def add_to_manifest(self, manifest_path):
        manifest = predix.app.Manifest(manifest_path)

        # Add this service to list of services
        manifest.add_service(self.service.name)

        # Add environment variables
        manifest.add_env_var('PREDIX_WEATHER_ZONE_ID',
                self.service.settings.data['zone']['http-header-value'])
        manifest.add_env_var('PREDIX_WEATHER_URI',
                self.service.settings.data['uri'])

        manifest.write_manifest()

    def get_oauth_scope(self):
        return self.service.settings.data['zone']['oauth-scope']

    def grant_client(self, client_id):
        """
        Grant the given client with any scopes or authorities
        needed to use this service.
        """
        scopes = ['openid']
        authorities = ['uaa.resource']

        scopes.append(self.get_oauth_scope())
        authorities.append(self.get_oauth_scope())

        self.service.uaa.uaac.update_client_grants(client_id, scope=scopes,
                authorities=authorities)

        return self.service.uaa.uaac.get_client(client_id)




import predix.admin.cf.api


class Org(object):
    """
    Operations and data for Cloud Foundry Organizations.
    """
    def __init__(self, *args, **kwargs):
        super(Org, self).__init__(*args, **kwargs)

        self.api = predix.admin.cf.api.API()
        self.name = self.api.config.get_organization_name()

    def _get_orgs(self):
        """
        Returns the organizations for the authenticated user.
        """
        return self.api.get('/v2/organizations')

    def get_orgs(self):
        """
        Returns a flat list of the names for the organizations
        user belongs.
        """
        orgs = []
        for resource in self._get_orgs()['resources']:
            orgs.append(resource['entity']['name'])

        return orgs

    def _get_apps(self):
        """
        Returns all of the apps in the organization.
        """
        return self.api.get('/v2/apps')

    def get_apps(self):
        """
        Returns a flat list of the names for the apps in
        the organization.
        """
        apps = []
        for resource in self._get_apps()['resources']:
            apps.append(resource['entity']['name'])

        return apps

    def has_app(self, app_name):
        """
        Simple test to see if we have a name conflict
        for the application.
        """
        return app_name in self.get_apps()




from setuptools import setup, find_packages

install_requires = [
        "python-dateutil",
        "PyYAML",
        "requests",
        "websocket"
    ]

setup_requires = [
        "coverage",
        "flake8",
        "nose",
        "pylint",
        "tox"
    ]

setup(
        name="predix",
        version="0.0.5",
        author="Jayson DeLancey",
        author_email="jayson.delancey@ge.com",
        description="Python Client SDK for Predix Services",
        setup_requires=setup_requires,
        install_requires=install_requires,
        package_data={},
        packages=find_packages(exclude=['test', 'test.*']),
        test_suite="test",
        entry_points={
            'console_scripts': [
                ]
        },
        tests_require=['tox'],
        keywords=['predix', 'ge', 'time', 'asset', 'analytics'],
        url="https://github.com/predixpy/predixpy",
        classifiers=[
            'Intended Audience :: Developers',
            'Natural Language :: English',
            'Programming Language :: Python :: 2.7',
            'License :: Other/Proprietary License',
            'Development Status :: 3 - Alpha']
    )




import os
import urlparse

import predix.app
import predix.security.uaa
import predix.admin.service


class TimeSeries(object):
    """
    Time Series provides persistence of data values over time.
    """
    def __init__(self, plan_name=None, name=None, uaa=None, *args, **kwargs):
        super(TimeSeries, self).__init__(*args, **kwargs)
        self.service_name = 'predix-timeseries'
        self.plan_name = plan_name or 'Free'
        self.service = predix.admin.service.PredixService(self.service_name,
                self.plan_name, name=name, uaa=uaa)

    def exists(self):
        """
        Returns whether or not this service already exists.
        """
        return self.service.exists()

    def create(self):
        """
        Create an instance of the Time Series Service with the typical
        starting settings.
        """
        self.service.create()

        uri = self.service.settings.data['ingest']['uri']
        os.environ['PREDIX_TIMESERIES_INGEST_URI'] = uri

        zone = self.get_ingest_zone_id()
        os.environ['PREDIX_TIMESERIES_INGEST_ZONE_ID'] = zone

        uri = self.service.settings.data['query']['uri']
        uri = urlparse.urlparse(uri)
        os.environ['PREDIX_TIMESERIES_QUERY_URI'] = uri.scheme + '://' + uri.netloc

        zone = self.get_query_zone_id()
        os.environ['PREDIX_TIMESERIES_QUERY_ZONE_ID'] = zone

    def grant_client(self, client_id, read=True, write=True):
        """
        Grant the given client id all the scopes and authorities
        needed to work with the timeseries service.
        """
        scopes = ['openid']
        authorities = ['uaa.resource']

        if write:
            for zone in self.service.settings.data['ingest']['zone-token-scopes']:
                scopes.append(zone)
                authorities.append(zone)

        if read:
            for zone in self.service.settings.data['query']['zone-token-scopes']:
                scopes.append(zone)
                authorities.append(zone)

        self.service.uaa.uaac.update_client_grants(client_id, scope=scopes,
                authorities=authorities)

        return self.service.uaa.uaac.get_client(client_id)

    def get_ingest_zone_id(self):
        """
        Returns the Predix-Zone-Id used for ingesting data with this
        service.
        """
        return self.service.settings.data['ingest']['zone-http-header-value']

    def get_query_zone_id(self):
        """
        Return the Predix-Zone-Id used for queries on data with this
        service.
        """
        return self.service.settings.data['query']['zone-http-header-value']

    def add_to_manifest(self, manifest_path):
        """
        Add details to the manifest that applications using
        this service may need to consume.
        """
        manifest = predix.app.Manifest(manifest_path)

        # Add this service to list of services
        manifest.add_service(self.service.name)

        # Add environment variables
        manifest.add_env_var('PREDIX_TIMESERIES_INGEST_URI',
                self.service.settings.data['ingest']['uri'])
        manifest.add_env_var('PREDIX_TIMESERIES_INGEST_ZONE_ID',
                self.get_ingest_zone_id())

        # Query URI has extra path we don't want
        uri = self.service.settings.data['query']['uri']
        uri = urlparse.urlparse(uri)
        manifest.add_env_var('PREDIX_TIMESERIES_QUERY_URI',
                uri.scheme + '://' + uri.netloc)
        manifest.add_env_var('PREDIX_TIMESERIES_QUERY_ZONE_ID',
                self.get_query_zone_id())

        manifest.write_manifest()
